import torch
from crtl.components.dual_encoder.pipeline.linker import Linker
from crtl.components.dual_encoder.pipeline.modules.embedder import (
    Embedder,
    EmbedderModule,
)
from crtl.components.dual_encoder.pipeline.preprocessing.medmentions_processing import (
    MedmentionsPreprocessor,
)
from crtl.components.dual_encoder.pipeline.preprocessing.preprocessor import (
    LKBPreprocessor,
)
from crtl.datasets.medmentions.parse_medmentions import MedMentionCorpus

from transformers import AutoTokenizer, AutoModel

from crtl.lkb.lkb import LexicalKnowledgeBase
from crtl.s3_file_utils.s3_file_utils import cached_path


def get_linker(tokenizer, lkb):
    mps_device = torch.device("mps")

    mention_embedder_module = EmbedderModule(
        AutoModel.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")
    )
    candidate_embedder_module = EmbedderModule(
        AutoModel.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")
    )

    mention_embedder = Embedder(tokenizer, mention_embedder_module)

    candidate_embedder = Embedder(tokenizer, candidate_embedder_module)

    linker = Linker(mention_embedder, candidate_embedder, lkb, mps_device)

    return linker


def get_medmentions_dataset(tokenizer, lkb, lkb_preprocessor):
    medmention_preprocessor = MedmentionsPreprocessor(
        tokenizer, length=50, eval_split=0.1, lkb_preprocessor=lkb_preprocessor
    )

    medmention_corpus = MedMentionCorpus.from_file(
        "/Users/coltonflowers/CRTL/src/crtl/datasets/medmentions/medmentions.txt"
    )

    preprocessed_dataset = medmention_preprocessor(medmention_corpus, lkb)

    return preprocessed_dataset


if __name__ == "__main__":

    tokenizer = AutoTokenizer.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")

    lkb = LexicalKnowledgeBase.from_disk(
        "/Users/coltonflowers/CRTL/UMLS2017_FULL.jsonl"
    )

    print("lkb loaded!")

    lkb_preprocessor = LKBPreprocessor(tokenizer, length=50)

    medmentions_dataset = get_medmentions_dataset(tokenizer, lkb, lkb_preprocessor)

    print("medmention_corpus loaded!")

    linker = get_linker(tokenizer, lkb)

    print("linker loaded!")

    # linker.train(medmentions_dataset)
