from crtl.components.dual_encoder.pipeline.training.dataloader_creator import (
    DataLoaderCreator,
)
from crtl.components.dual_encoder.pipeline.modules.embedder import Embedder
from crtl.components.dual_encoder.pipeline.training.train import (
    EntityLinkerTrainer,
    PreproccessedDataSet,
)
from crtl.lkb.lkb import LexicalKnowledgeBase


class Linker:
    def __init__(
        self,
        candidate_embedder: Embedder,
        mention_embedder: Embedder,
        lkb: LexicalKnowledgeBase,
        device,
    ):
        self.candidate_embedder = candidate_embedder
        self.mention_embedder = mention_embedder
        self.lkb = lkb
        self.device = device

    def initialize(
        self,
        batch_size: int = 1000,
        num_epochs: int = 10,
        learning_rate: float = 1e-5,  # Recommended by https://arxiv.org/pdf/2103.05028.pdf
        verbose: bool = True,
        num_random_negatives: int = 5,
        num_hard_negatives: int = 5,
    ):
        self._trainer = EntityLinkerTrainer(
            self.candidate_embedder.embedder_module.to(self.device),
            self.mention_embedder.embedder_module.to(self.device),
            self.device,
            batch_size,
            num_epochs,
            learning_rate,
            verbose,
            num_random_negatives,
            num_hard_negatives,
        )

    def train(self, dataset: PreproccessedDataSet):
        self._trainer.train(dataset)


if __name__ == "__main__":
    tokenizer = AutoTokenizer.from_pretrained("sentence-transformers/all-MiniLM-L6-v2")
    MedmentionsPreprocessor()
    training_data = DataLoaderCreator()
    eval_data = DataLoaderCreator()

    # @classmethod
    # def from_pretrained_bert(
    #     cls,
    #     mention_pretrained_hf_bert_path: str = "sentence-transformers/all-MiniLM-L6-v2",
    #     candidate_pretrained_hf_bert_path: str = "sentence-transformers/all-MiniLM-L6-v2",
    # ):
    #     mention_embedder = Embedder.from_pretrained(mention_pretrained_hf_bert_path)
    #     candidate_embedder = Embedder.from_pretrained(candidate_pretrained_hf_bert_path)
    #     return cls(mention_embedder, candidate_embedder)
