import torch


class Scorer:
    def recall_at_n_score(
        self,
        gold_candidate_ids: torch.Tensor,
        similarity_scores: torch.Tensor,
        n: int = 1,
    ) -> float:
        # gold_candidate_ids (num_mentions)
        # similarity_scores (num_mentions x num_candidates)
        top_n = similarity_scores.argsort(dim=1, descending=True)[:, :n]
        gold_in_top_n = (
            gold_candidate_ids.to(torch.device("cpu")).unsqueeze(1) == top_n
        ).any(1)
        return torch.sum(gold_in_top_n).detach().item() / len(gold_candidate_ids)

    def average_rank_of_gold_score(
        self,
        gold_candidate_ids: torch.Tensor,
        similarity_scores: torch.Tensor,
        n: int = 1,
    ):
        sorted_similarities = similarity_scores.argsort(dim=1, descending=True)
        ranks = (gold_candidate_ids.unsqueeze(1) == sorted_similarities).nonzero(
            as_tuple=False
        )[:, 1]
        return torch.sum(ranks) / len(ranks)
